##### The explanation and derivation of equations correponding to this code are provided in the paper titled
##### "Simulation Estimation of Mixed Discrete Choice Models Using Randomized and Scrambled Halton Sequences " by Dr. Bhat
##### The paper is available on Dr. Bhat's website <http://www.ce.utexas.edu/prof/bhat/halton.html>

##### Change working directory to the one in which this code resides
setwd("C:/workspace/VFC_Package_Beta/Halton_R")

##### Vector of prime numbers #####
t1 <- c(2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61);
t2 <- c(67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151);
t3 <- c(157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223);
prim <<- matrix(c(t1, t2, t3),nrow=1);


##########################################################################
# total number of digits in digitization; since the lowest prime is 2,
# using maxdigit = 22 will be okay for generating halton sequences of
# length less than or equal to 2^22 = 4194304 numbers; if you want more
# than these many numbers, can increase maxdigit 
maxdigit <<- 22;


#############################################################################
# Should feed in values of wdata, sbrat, xbrat and xiden
# through main program; wdata is global with name of dataset to 
# which Halton numbers will be written; sbrat is global that determines
# if standard (sbrat=FALSE) or Bratten-Weller scrambled (sbrat=TRUE) sequence
# is desired; xbrat and xiden are matrices that need to be loaded thru main
# program before call to Halton 
sbrat <<- TRUE;

##### Output file name to which you want the halton sequence written #####
wdata <<- 1;
Output_File <<- "Test_5p_synhhld.csv";

##### Number of draws #####
# Develop a halton matrix (hm) of size nrep (number of points desired) x dim
# (number of rows) 
draws <- 10*8259;

##### Number of dimensions #####
dim <- 14;


##### Load permutation matrices #####
xbrat <<- read.table("xbrat.csv", header=F, sep=",") 
xiden <<- read.table("xiden.csv", header=F, sep=",") 


############################################################################
# Initializing routine 
creat <- function(s) {
  
  a <- matrix(0,nrow=s,ncol=maxdigit);
  a[,1] <- matrix(1,nrow=s,ncol=1);
  b <- matrix(1,nrow=s,ncol=1);
  
  return(list(a,b));
}


############################################################################
# Compute the radical inverse (Equation 2 of accompanying paper) 
inverse <- function(s,c,a,b,f) {
  
  m <- 1;
  gg <- matrix(0, nrow=1, ncol=s);  
  while (m <= s) {
    l <- 1;
    xx <- c[m];
    while (l <= b[m]) {
      gg[m] <- (f[m,(a[m,l]+1)]/xx)+(gg[m]);
      xx <- xx*c[m];
      
      l <- l+1;
    }
    
    m <- m+1;
  }
  
  return(gg);
}


############################################################################
# Heart of program; determines coefficients (the b's in equation 1 of
# accompanying paper) and number of digits (L) for each number from b and L
# values of previous number 
digitize <- function(s,c,a,b) {
  m <- 1;
  x <- NULL;
  while (m <= s) {
    l <- 1;
    ret <- 1;
    
    while (ret == 1) {
      x <- a[m,l] != (c[m]-1);
      ret <- ret - x;
      a[m,l] <- (a[m,l]+1)*(x==1);
      b[m] <- ((l-1) == b[m]) + b[m];
      
      l <- l+1;
    }
    
    m <- m+1;
  }
  
  return(list(a,b));
}


halton <- function(n,s) {
  
  zz <- file(Output_File,"w");
  h <- NULL;
  primer <- prim[1,1:s];
  
  table <- NULL;
  if (sbrat) {
    table <- xbrat;
  }
  else {
    table <- xiden;
  }
  
  temp <- creat(s);
  count <- temp[[1]];
  digit <- temp[[2]];
  h <- inverse(s,primer,count,digit,table);
  writeLines(toString(h),con=zz,sep="\n");
  
  i <- 2;
  while(i <= n) {
    temp <- digitize(s,primer,count,digit);
    count <- temp[[1]];
    digit <- temp[[2]];
    
    h1 <- inverse(s,primer,count,digit,table);
    writeLines(toString(h1),con=zz,sep="\n");
#    h <- rbind(h,h1);
#    
#     if(wdata == 1) {
#       y <- writer(fout, h1);
#     }
#     else {
#       h <- rbind(h,h1);
#     }
    
    i <- i+1;
  }
  
  close(zz);
}

ptm <- proc.time();
halton(draws, dim);
print(proc.time() - ptm);


